package main

import (
	"context"
	"os"
	"os/signal"

	"github.com/sirupsen/logrus"

	"git.janky.solutions/finn/go-project-template/config"
	"git.janky.solutions/finn/go-project-template/db"
	"git.janky.solutions/finn/go-project-template/httpserver"
)

func main() {
	run()
}

func run() {
	if err := config.Load(); err != nil {
		logrus.WithError(err).Fatal("error loading config")
	}
	logrus.SetLevel(logrus.DebugLevel)

	ctx, cancel := signal.NotifyContext(context.Background(), os.Interrupt)
	defer cancel()

	if err := db.Migrate(ctx); err != nil {
		logrus.WithError(err).Fatal("error migrating database")
	}

	go httpserver.ListenAndServe()

	<-ctx.Done()

	if err := httpserver.Shutdown(ctx); err != nil {
		logrus.WithError(err).Error("error shutting down web server")
	}
}
